package com.segment.analyticsreactnativepluginsignals

import android.view.View
import android.view.ViewGroup
import android.view.ViewParent
import androidx.core.view.children
import com.facebook.react.bridge.Arguments
import com.facebook.react.bridge.ReactApplicationContext
import com.facebook.react.bridge.WritableMap
import com.facebook.react.touch.JSResponderHandler
import com.facebook.react.touch.OnInterceptTouchEventListener
import com.facebook.react.uimanager.NativeViewHierarchyManager
import com.facebook.react.uimanager.UIImplementation
import com.facebook.react.uimanager.UIManagerModule
import com.facebook.react.uimanager.UIViewOperationQueue
import com.facebook.react.views.text.ReactTextView
import java.lang.ref.WeakReference


class SignalsUIToolkitTrackingPlugin(
  private val emit: (event: String, params: WritableMap) -> Unit
) {

  var uiManagerModuleRef: WeakReference<UIManagerModule>? = null

  var initialized = false

  fun autoTracking(reactContext: ReactApplicationContext) {
    if (initialized) return
    swizzle(reactContext)
    initialized = true
  }

  fun swizzle(reactContext: ReactApplicationContext) {
    val uiManagerModule = reactContext.getNativeModule(UIManagerModule::class.java)
    uiManagerModule?.let {
      var field = UIImplementation::class.java.getDeclaredField("mOperationsQueue")
      field.isAccessible = true
      val queue = field.get(it.uiImplementation) as UIViewOperationQueue
      field = UIViewOperationQueue::class.java.getDeclaredField("mNativeViewHierarchyManager")
      field.isAccessible = true
      val manager = field.get(queue) as NativeViewHierarchyManager
      field = NativeViewHierarchyManager::class.java.getDeclaredField("mJSResponderHandler")
      field.isAccessible = true
      val listener = field.get(manager)
      if (listener !is JSResponderHandlerAdapter && listener is OnInterceptTouchEventListener) {
        field.set(manager, JSResponderHandlerAdapter(this::onViewClicked))
      }
      uiManagerModuleRef = WeakReference(it)
    }
  }

  private fun onViewClicked(tag: Int) {
    val view = uiManagerModuleRef?.get()?.resolveView(tag) ?: return

    val sb = extractData(view, StringBuilder())
    if (sb.isEmpty()) {
      if (view.contentDescription != null) {
        sb.append(view.contentDescription)
      }
      else {
        sb.append("Unknown Button")
      }
    }

    emit("InteractionSignal", Arguments.createMap().apply {
      putString("component", sb.toString())
    })
  }

  private fun extractData(view: View, sb: StringBuilder): StringBuilder {
    if (view is ReactTextView) {
      if (sb.isNotEmpty()) sb.append("; ")
      sb.append(view.text)
    }
    else if (view is ViewGroup) {
      for (v in view.children) {
        extractData(v, sb)
      }
    }

    return sb
  }

  class JSResponderHandlerAdapter(
    private val onViewClicked: (Int) -> Unit
  ): JSResponderHandler() {
    override fun setJSResponder(tag: Int, viewParentBlockingNativeResponder: ViewParent?) {
      onViewClicked(tag)
      super.setJSResponder(tag, viewParentBlockingNativeResponder)
    }
  }
}
