package com.segment.analyticsreactnativepluginsignals

import android.annotation.SuppressLint
import android.graphics.Matrix
import android.graphics.PointF
import android.view.View
import android.view.ViewGroup
import com.facebook.common.logging.FLog
import com.facebook.react.bridge.UiThreadUtil
import com.facebook.react.touch.ReactHitSlopView
import com.facebook.react.uimanager.PointerEvents
import com.facebook.react.uimanager.ReactCompoundView
import com.facebook.react.uimanager.ReactCompoundViewGroup
import com.facebook.react.uimanager.ReactOverflowViewWithInset
import com.facebook.react.uimanager.ReactPointerEventsView
import com.facebook.react.uimanager.ReactZIndexedViewGroup
import com.facebook.react.uimanager.common.UIManagerType
import com.facebook.react.uimanager.common.ViewUtil
import java.util.EnumSet
import java.util.Objects


object TouchTargetHelper {
  private val mEventCoords = FloatArray(2)
  private val mTempPoint = PointF()
  private val mMatrixTransformCoords = FloatArray(2)
  private val mInverseMatrix = Matrix()
  fun findTargetTagForTouch(eventX: Float, eventY: Float, viewGroup: ViewGroup): Int {
    return findTargetTagAndCoordinatesForTouch(
      eventX,
      eventY,
      viewGroup,
      mEventCoords,
      null as IntArray?
    )
  }

  fun findTargetTagForTouch(
    eventX: Float,
    eventY: Float,
    viewGroup: ViewGroup,
    nativeViewId: IntArray?
  ): Int {
    return findTargetTagAndCoordinatesForTouch(
      eventX,
      eventY,
      viewGroup,
      mEventCoords,
      nativeViewId
    )
  }

  fun findTargetTagAndCoordinatesForTouch(
    eventX: Float,
    eventY: Float,
    viewGroup: ViewGroup,
    viewCoords: FloatArray,
    nativeViewTag: IntArray?
  ): Int {
    UiThreadUtil.assertOnUiThread()
    var targetTag = viewGroup.id
    viewCoords[0] = eventX
    viewCoords[1] = eventY
    val nativeTargetView =
      findTouchTargetViewWithPointerEvents(viewCoords, viewGroup, null)
    if (nativeTargetView != null) {
      val reactTargetView = findClosestReactAncestor(nativeTargetView)
      if (reactTargetView != null) {
        if (nativeViewTag != null) {
          nativeViewTag[0] = reactTargetView.id
        }
        targetTag = getTouchTargetForView(
          reactTargetView,
          viewCoords[0], viewCoords[1]
        )
      }
    }
    return targetTag
  }

  fun findTouchTargetView(eventX: Float, eventY: Float, viewGroup: ViewGroup): View? {
    mEventCoords[0] = eventX
    mEventCoords[1] = eventY
    var pathAccumulator: MutableList<ViewTarget> = mutableListOf()
    return findTouchTargetViewWithPointerEvents(mEventCoords, viewGroup, pathAccumulator)
  }

  @SuppressLint("ResourceType")
  fun findTargetPathAndCoordinatesForTouch(
    eventX: Float,
    eventY: Float,
    viewGroup: ViewGroup,
    viewCoords: FloatArray
  ): List<ViewTarget> {
    UiThreadUtil.assertOnUiThread()
    viewCoords[0] = eventX
    viewCoords[1] = eventY
    var pathAccumulator: MutableList<ViewTarget> = mutableListOf()
    val targetView =
      findTouchTargetViewWithPointerEvents(viewCoords, viewGroup, pathAccumulator)
    if (targetView != null) {
      var reactTargetView = targetView
      var firstReactAncestor: Int
      firstReactAncestor = 0
      while (reactTargetView != null && reactTargetView.id <= 0) {
        reactTargetView = reactTargetView.parent as View
        ++firstReactAncestor
      }
      if (firstReactAncestor > 0) {
        pathAccumulator = pathAccumulator.subList(
          firstReactAncestor,
          (pathAccumulator as List<*>).size
        )
      }
      val targetTag = getTouchTargetForView(
        reactTargetView,
        viewCoords[0], viewCoords[1]
      )
      if (targetTag != reactTargetView!!.id) {
        pathAccumulator.add(0, ViewTarget(targetTag, null))
      }
    }
    return pathAccumulator
  }

  @SuppressLint("ResourceType")
  private fun findClosestReactAncestor(view: View): View? {
    var view: View? = view
    while (view != null && view.id <= 0) {
      view = view.parent as View
    }
    return view
  }

  private fun findTouchTargetView(
    eventCoords: FloatArray,
    view: View,
    allowReturnTouchTargetTypes: EnumSet<TouchTargetReturnType>,
    pathAccumulator: MutableList<ViewTarget>?
  ): View? {
    if (allowReturnTouchTargetTypes.contains(TouchTargetReturnType.CHILD) && view is ViewGroup) {
      val viewGroup = view
      if (!isTouchPointInView(eventCoords[0], eventCoords[1], view)) {
        if (view is ReactOverflowViewWithInset) {
          if (ViewUtil.getUIManagerType(view.getId()) == UIManagerType.FABRIC && !isTouchPointInViewWithOverflowInset(
              eventCoords[0], eventCoords[1], view
            )
          ) {
            return null
          }
          val overflow = (view as ReactOverflowViewWithInset).overflow
          if ("hidden" == overflow || "scroll" == overflow) {
            return null
          }
        }
        if (viewGroup.clipChildren) {
          return null
        }
      }
      val childrenCount = viewGroup.childCount
      val zIndexedViewGroup = if (viewGroup is ReactZIndexedViewGroup) viewGroup else null
      for (i in childrenCount - 1 downTo 0) {
        val childIndex = zIndexedViewGroup?.getZIndexMappedChildIndex(i) ?: i
        val child = viewGroup.getChildAt(childIndex)
        val childPoint = mTempPoint
        getChildPoint(
          eventCoords[0],
          eventCoords[1], viewGroup, child, childPoint
        )
        val restoreX = eventCoords[0]
        val restoreY = eventCoords[1]
        eventCoords[0] = childPoint.x
        eventCoords[1] = childPoint.y
        val targetView = findTouchTargetViewWithPointerEvents(eventCoords, child, pathAccumulator)
        if (targetView != null) {
          return targetView
        }
        eventCoords[0] = restoreX
        eventCoords[1] = restoreY
      }
    }
    return if (allowReturnTouchTargetTypes.contains(TouchTargetReturnType.SELF) && isTouchPointInView(
        eventCoords[0], eventCoords[1], view
      )
    ) view else null
  }

  private fun isTouchPointInView(x: Float, y: Float, view: View): Boolean {
    return if (view is ReactHitSlopView && (view as ReactHitSlopView).hitSlopRect != null) {
      val hitSlopRect = (view as ReactHitSlopView).hitSlopRect
      x >= (-hitSlopRect!!.left).toFloat() && x < (view.width + hitSlopRect.right).toFloat() && y >= (-hitSlopRect.top).toFloat() && y < (view.height + hitSlopRect.bottom).toFloat()
    } else {
      x >= 0.0f && x < view.width.toFloat() && y >= 0.0f && y < view.height.toFloat()
    }
  }

  private fun isTouchPointInViewWithOverflowInset(x: Float, y: Float, view: View): Boolean {
    return if (view !is ReactOverflowViewWithInset) {
      false
    } else {
      val overflowInset = (view as ReactOverflowViewWithInset).overflowInset
      x >= overflowInset.left.toFloat() && x < (view.width - overflowInset.right).toFloat() && y >= overflowInset.top.toFloat() && y < (view.height - overflowInset.bottom).toFloat()
    }
  }

  private fun getChildPoint(
    x: Float,
    y: Float,
    parent: ViewGroup,
    child: View,
    outLocalPoint: PointF
  ) {
    var localX = x + parent.scrollX.toFloat() - child.left.toFloat()
    var localY = y + parent.scrollY.toFloat() - child.top.toFloat()
    val matrix = child.getMatrix()
    if (!matrix.isIdentity) {
      val localXY = mMatrixTransformCoords
      localXY[0] = localX
      localXY[1] = localY
      val inverseMatrix = mInverseMatrix
      matrix.invert(inverseMatrix)
      inverseMatrix.mapPoints(localXY)
      localX = localXY[0]
      localY = localXY[1]
    }
    outLocalPoint[localX] = localY
  }

  private fun findTouchTargetViewWithPointerEvents(
    eventCoords: FloatArray,
    view: View,
    pathAccumulator: MutableList<ViewTarget>?
  ): View? {
    var pointerEvents =
      if (view is ReactPointerEventsView) (view as ReactPointerEventsView).pointerEvents else PointerEvents.AUTO
    if (!view.isEnabled) {
      if (pointerEvents == PointerEvents.AUTO) {
        pointerEvents = PointerEvents.BOX_NONE
      } else if (pointerEvents == PointerEvents.BOX_ONLY) {
        pointerEvents = PointerEvents.NONE
      }
    }
    return if (pointerEvents == PointerEvents.NONE) {
      null
    } else {
      val result: View?
      if (pointerEvents == PointerEvents.BOX_ONLY) {
        result = findTouchTargetView(
          eventCoords, view, EnumSet.of(
            TouchTargetReturnType.SELF
          ), pathAccumulator
        )
        if (result != null && pathAccumulator != null) {
          pathAccumulator.add(ViewTarget(view.id, view))
        }
        result
      } else if (pointerEvents == PointerEvents.BOX_NONE) {
        result = findTouchTargetView(
          eventCoords, view, EnumSet.of(
            TouchTargetReturnType.CHILD
          ), pathAccumulator
        )
        if (result != null) {
          pathAccumulator?.add(ViewTarget(view.id, view))
          result
        } else {
          if (view is ReactCompoundView && isTouchPointInView(
              eventCoords[0], eventCoords[1], view
            )
          ) {
            val reactTag = (view as ReactCompoundView).reactTagForTouch(
              eventCoords[0],
              eventCoords[1]
            )
            if (reactTag != view.id) {
              pathAccumulator?.add(ViewTarget(view.id, view))
              return view
            }
          }
          null
        }
      } else {
        if (pointerEvents != PointerEvents.AUTO) {
          FLog.w(
            "ReactNative",
            "Unknown pointer event type: $pointerEvents"
          )
        }
        if (view is ReactCompoundViewGroup && isTouchPointInView(
            eventCoords[0], eventCoords[1], view
          ) && (view as ReactCompoundViewGroup).interceptsTouchEvent(
            eventCoords[0], eventCoords[1]
          )
        ) {
          pathAccumulator?.add(ViewTarget(view.id, view))
          view
        } else {
          result = findTouchTargetView(
            eventCoords, view, EnumSet.of(
              TouchTargetReturnType.SELF, TouchTargetReturnType.CHILD
            ), pathAccumulator
          )
          if (result != null && pathAccumulator != null) {
            pathAccumulator.add(ViewTarget(view.id, view))
          }
          result
        }
      }
    }
  }

  private fun getTouchTargetForView(targetView: View?, viewX: Float, viewY: Float): Int {
    return if (targetView is ReactCompoundView) (targetView as ReactCompoundView).reactTagForTouch(
      viewX,
      viewY
    ) else targetView!!.id
  }

  class ViewTarget(val viewId: Int, val view: View?) {

    override fun equals(o: Any?): Boolean {
      return if (o === this) {
        true
      } else if (o !is ViewTarget) {
        false
      } else {
        o.viewId == viewId
      }
    }

    override fun hashCode(): Int {
      return Objects.hashCode(viewId)
    }
  }

  private enum class TouchTargetReturnType {
    SELF,
    CHILD
  }
}
