import React

//
//  SignalsTapTracking.swift
//  Shopify
//
//  Created by Brandon Sneed on 9/27/22.
//

#if os(iOS) || os(tvOS) || os(visionOS) || targetEnvironment(macCatalyst)

import Foundation
import UIKit

class SignalsTapTracking {
    init() {
        setupUIKitHooks()
    }

    internal func setupUIKitHooks() {
        swizzle(forClass: RCTUIManager.self,
                original: NSSelectorFromString("setJSResponder:blockNativeResponder:"),
                new: #selector(RCTUIManager.swizzled_setJSResponder(_:blockNativeResponder:))
        )
    }
}

extension SignalsTapTracking {
    private func swizzle(forClass: AnyClass, original: Selector, new: Selector) {
        guard let originalMethod = class_getInstanceMethod(forClass, original) else { return }
        guard let swizzledMethod = class_getInstanceMethod(forClass, new) else { return }
        method_exchangeImplementations(originalMethod, swizzledMethod)
    }
}

extension RCTUIManager {

    @objc dynamic func swizzled_setJSResponder(_ reactTag: NSNumber, blockNativeResponder: Bool) {
        DispatchQueue.main.sync {
            if let view = view(forReactTag: reactTag) {
                let sb = extractData(view, stringBuilder: NSMutableString())
                if (sb.length == 0) {
                    if let text = view.accessibilityLabel {
                        sb.append(text)
                    }
                    else {
                        sb.append("Unknown Button")
                    }
                }
                
                AnalyticsReactNativePluginSignals.emitEventToJavaScript(
                  eventName: "InteractionSignal",
                  eventBody: ["component": String(sb)]
                )
            }
        }
        // Call the original method
        swizzled_setJSResponder(reactTag, blockNativeResponder: blockNativeResponder)
    }
    
    func extractData(_ view: UIView, stringBuilder: NSMutableString) -> NSMutableString {
        if let textView = view as? RCTTextView {
            if let text = textView.accessibilityLabel {
                if (stringBuilder.length > 0) {
                    stringBuilder.append("; ")
                }
                stringBuilder.append(text)
            }
        }
        else {
            for subview in view.subviews {
                _ = extractData(subview, stringBuilder: stringBuilder)
            }
        }
        
        return stringBuilder
    }

}

#endif
