"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SegmentBroadcaster = void 0;
var _analyticsReactNative = require("@segment/analytics-react-native");
const MAX_EVENTS_PER_BATCH = 100;
const MAX_PAYLOAD_SIZE_IN_KB = 500;
class SegmentBroadcaster {
  sendEvents = async events => {
    if (events.length === 0) {
      return Promise.resolve();
    }

    // We're not sending events until Segment has loaded all settings
    // await this.settingsPromise;

    const config = this._analytics.getConfig() ?? _analyticsReactNative.defaultConfig;
    const chunkedEvents = (0, _analyticsReactNative.chunk)(events, config.maxBatchSize ?? MAX_EVENTS_PER_BATCH, MAX_PAYLOAD_SIZE_IN_KB);
    let sentEvents = [];
    let numFailedEvents = 0;
    await Promise.all(chunkedEvents.map(async batch => {
      try {
        const res = await (0, _analyticsReactNative.uploadEvents)({
          writeKey: config.writeKey,
          url: 'https://signals.segment.io/v1/b',
          events: batch
        });
        (0, _analyticsReactNative.checkResponseForErrors)(res);
        sentEvents = sentEvents.concat(batch);
      } catch (e) {
        this._analytics.reportInternalError((0, _analyticsReactNative.translateHTTPError)(e));
        this._analytics.logger.warn(e);
        numFailedEvents += batch.length;
      } finally {
        await this._flushPlugin.dequeue(sentEvents);
      }
    }));
    if (sentEvents.length) {
      if (config.debug === true) {
        this._analytics.logger.info(`Sent ${sentEvents.length} signlas`);
      }
    }
    if (numFailedEvents) {
      this._analytics.logger.error(`Failed to send ${numFailedEvents} signals.`);
    }
    return Promise.resolve();
  };
  _flushPlugin = new _analyticsReactNative.QueueFlushingPlugin(this.sendEvents, 'signals');
  constructor(analytics) {
    this._analytics = analytics;
    this._flushPlugin.configure(analytics);
  }
  added(signal) {
    // TODO: add obfuscation
    const userInfo = this._analytics.userInfo.get();
    this._flushPlugin.execute(this.createMiniTrackEvent(userInfo.anonymousId, userInfo.userId, signal));
  }
  relay() {
    this._flushPlugin.flush();
  }
  createMiniTrackEvent(anonymousId, userId, properties) {
    const miniEvent = (0, _analyticsReactNative.createTrackEvent)({
      event: 'Segment Signal Generated',
      properties: properties
    });
    miniEvent.anonymousId = anonymousId;
    miniEvent.userId = userId;
    miniEvent.timestamp = new Date().toISOString();
    miniEvent.context = {};
    miniEvent.integrations = {};
    return miniEvent;
  }
}
exports.SegmentBroadcaster = SegmentBroadcaster;
//# sourceMappingURL=segment-broadcaster.js.map