"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SignalsNetworkTracking = void 0;
var _analyticsReactNative = require("@segment/analytics-react-native");
var _signals = require("../signals");
var _types = require("../types");
const originalFetch = global.fetch;
class SignalsNetworkTracking extends _analyticsReactNative.UtilityPlugin {
  type = _analyticsReactNative.PluginType.utility;
  configure(analytics) {
    super.configure(analytics);
    global.fetch = async (url, data) => {
      const isSegmentEndpoint = this.containsSpecificUrl(url, 'segment.io');
      const urlStr = this.urlToString(url);

      // Intercept the request
      if (!isSegmentEndpoint) {
        _signals.Signals.emit(new _types.Network.Signal(_types.Network.Action.Request, urlStr, JSON.stringify(data)));
      }

      // Perform the original fetch
      const response = await originalFetch(url, data);

      // Intercept the response
      const clonedResponse = response.clone();
      const responseBody = await clonedResponse.json();
      if (!isSegmentEndpoint) {
        _signals.Signals.emit(new _types.Network.Signal(_types.Network.Action.Response, urlStr, responseBody));
      }
      return response;
    };
  }
  containsSpecificUrl(value, specificUrl) {
    if (typeof value === 'string') {
      return value.includes(specificUrl);
    } else if (value instanceof URL) {
      return value.href.includes(specificUrl);
    } else if (value instanceof Request) {
      return value.url.includes(specificUrl);
    }
    return false;
  }
  urlToString(value) {
    if (typeof value === 'string') {
      return value;
    } else if (value instanceof URL) {
      return value.href;
    } else if (value instanceof Request) {
      return value.url;
    }
    return '';
  }
}
exports.SignalsNetworkTracking = SignalsNetworkTracking;
//# sourceMappingURL=signals-network-tracking.js.map