"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SignalsPlugin = exports.Signals = void 0;
var _analyticsReactNative = require("@segment/analytics-react-native");
var _types = require("./types");
var _compat = require("./compat");
var _analyticsReactNativePluginLive = require("@segment/analytics-react-native-plugin-live");
class SignalsPlugin extends _analyticsReactNative.Plugin {
  type = _analyticsReactNative.PluginType.after;
  _config = {
    maximumBufferSize: 1000,
    broadcastInterval: 60
  };
  set config(value) {
    this._config = value;
    this.updateJSConfiguration();
    this.updateNativeConfiguration();
  }
  get broadcasters() {
    return this._config.broadcasters ?? [];
  }
  _queuedSignals = [];
  _ready = false;
  _counter = 0;
  _timerStarted = false;
  _signals = new _compat.SignalsJS();
  _runtime = {};
  get runtime() {
    return {
      ...this._runtime
    };
  }
  get anonymousId() {
    return this.analytics?.userInfo.get().anonymousId ?? "";
  }
  get nextIndex() {
    return this._signals.getNextIndex();
  }
  get buffer() {
    return this._signals.signalBuffer;
  }
  get jsSignals() {
    return this._signals;
  }
  constructor() {
    super();
  }
  prepare(compatLayer) {
    this._compatLayer = compatLayer;
    this._runtime = {
      'SignalType': new Proxy(_types.SignalType, new _analyticsReactNativePluginLive.SafeProxy()),
      'RawSignal': new Proxy(_types.RawSignal, new _analyticsReactNativePluginLive.SafeProxy()),
      'NavigationAction': new Proxy(_types.Navigation.Action, new _analyticsReactNativePluginLive.SafeProxy()),
      'NavigationData': new Proxy(_types.Navigation.Data, new _analyticsReactNativePluginLive.SafeProxy()),
      'NavigationSignal': new Proxy(_types.Navigation.Signal, new _analyticsReactNativePluginLive.SafeProxy()),
      'InteractionData': new Proxy(_types.Interaction.Data, new _analyticsReactNativePluginLive.SafeProxy()),
      'InteractionSignal': new Proxy(_types.Interaction.Signal, new _analyticsReactNativePluginLive.SafeProxy()),
      'NetworkAction': new Proxy(_types.Network.Action, new _analyticsReactNativePluginLive.SafeProxy()),
      'NetworkData': new Proxy(_types.Network.Data, new _analyticsReactNativePluginLive.SafeProxy()),
      'NetworkSignal': new Proxy(_types.Network.Signal, new _analyticsReactNativePluginLive.SafeProxy()),
      'LocalDataAction': new Proxy(_types.LocalData.Action, new _analyticsReactNativePluginLive.SafeProxy()),
      'LocalData': new Proxy(_types.LocalData.Data, new _analyticsReactNativePluginLive.SafeProxy()),
      'LocalDataSignal': new Proxy(_types.LocalData.Signal, new _analyticsReactNativePluginLive.SafeProxy()),
      'EventType': new Proxy(_types.Instrumentation.EventType, new _analyticsReactNativePluginLive.SafeProxy()),
      'InstrumentationData': new Proxy(_types.Instrumentation.Data, new _analyticsReactNativePluginLive.SafeProxy()),
      'InstrumentationSignal': new Proxy(_types.Instrumentation.Signal, new _analyticsReactNativePluginLive.SafeProxy()),
      'Signals': new Proxy(SignalsPlugin, new _analyticsReactNativePluginLive.SafeProxy()),
      'signals': this._signals
    };
  }
  readyToStart() {
    this._ready = true;
    this.updateJSConfiguration();
    this.updateNativeConfiguration();
    this.replayQueuedSignals();
  }
  static get instance() {
    return this._instance || (this._instance = new this());
  }
  configure(analytics) {
    this.analytics = analytics;
    const liveplugin = analytics.getPlugins().find(p => p instanceof _analyticsReactNativePluginLive.LivePlugins);
    if (liveplugin) {
      liveplugin.addDependent(this);
    }
  }
  execute(event) {
    if (event.context?.__eventOrigin?.type == 'js') return;
    this.emit(new _types.Instrumentation.Signal(event));
    return event;
  }
  emit(signal) {
    if (!this._ready) {
      this._queuedSignals.push(signal);
      return;
    }
    this.broadcasters.forEach(b => {
      b.added(signal);
    });
    this._signals.add(signal);
    if (this._compatLayer) {
      const params = {
        ...this.runtime,
        'signal': signal
      };
      (0, _analyticsReactNativePluginLive.sandboxExecute)(this._compatLayer, 'processSignal(signal);', params);
    }
    if (this._counter > this._config.maximumBufferSize) {
      this._counter = 0;
      this.broadcasters.forEach(b => {
        b.relay();
      });
    }
  }
  replayQueuedSignals() {
    while (this._queuedSignals.length) {
      var signal = this._queuedSignals.shift();
      if (signal) {
        signal.index = this.nextIndex;
        signal.anonymousId = this.anonymousId;
        this.emit(signal);
      }
    }
  }
  updateJSConfiguration() {
    this._signals.maxBufferSize = this._config.maximumBufferSize;
  }
  updateNativeConfiguration() {
    this.schedule();
  }
  schedule() {
    if (this._timerStarted) return;
    this._timerStarted = true;
    this._timer = setInterval(() => {
      this.broadcasters.forEach(b => {
        b.relay();
      });
    }, this._config.broadcastInterval * 1000);
  }

  // @ts-ignore
  unschedule() {
    if (this._timerStarted) {
      clearTimeout(this._timer);
    }
  }
}
exports.SignalsPlugin = SignalsPlugin;
const Signals = exports.Signals = SignalsPlugin.instance;
//# sourceMappingURL=signals.js.map