"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SignalType = exports.RawSignal = exports.Network = exports.Navigation = exports.LocalData = exports.Interaction = exports.Instrumentation = void 0;
var _signals = require("./signals");
class RawSignal {}
exports.RawSignal = RawSignal;
let SignalType = exports.SignalType = /*#__PURE__*/function (SignalType) {
  SignalType["Interaction"] = "interaction";
  SignalType["Navigation"] = "navigation";
  SignalType["Network"] = "network";
  SignalType["LocalData"] = "localData";
  SignalType["Instrumentation"] = "instrumentation";
  SignalType["UserDefined"] = "userDefined";
  return SignalType;
}({});
let Navigation = exports.Navigation = void 0;
(function (_Navigation) {
  let Action = /*#__PURE__*/function (Action) {
    Action["Forward"] = "forward";
    Action["Backward"] = "backward";
    Action["Modal"] = "modal";
    Action["Entering"] = "entering";
    Action["Leaving"] = "leaving";
    Action["Page"] = "page";
    Action["Popup"] = "popup";
    return Action;
  }({});
  _Navigation.Action = Action;
  class Data {
    constructor(action, screen) {
      this.action = action;
      this.screen = screen;
    }
  }
  _Navigation.Data = Data;
  class Signal extends RawSignal {
    constructor(action, screen) {
      super();
      this.data = new Data(action, screen);
    }
    anonymousId = _signals.Signals.anonymousId;
    type = SignalType.Navigation;
    timestamp = new Date().toISOString();
    index = _signals.Signals.nextIndex;
  }
  _Navigation.Signal = Signal;
})(Navigation || (exports.Navigation = Navigation = {}));
let Interaction = exports.Interaction = void 0;
(function (_Interaction) {
  class Data {
    constructor(component, title, data) {
      this.component = component;
      this.title = title;
      this.data = data;
    }
  }
  _Interaction.Data = Data;
  class Signal extends RawSignal {
    constructor(control, title, data) {
      super();
      const json = data === null || data === undefined ? undefined : JSON.parse(data);
      this.data = new Data(control, title, json);
    }
    anonymousId = _signals.Signals.anonymousId;
    type = SignalType.Interaction;
    timestamp = new Date().toISOString();
    index = _signals.Signals.nextIndex;
  }
  _Interaction.Signal = Signal;
})(Interaction || (exports.Interaction = Interaction = {}));
let Network = exports.Network = void 0;
(function (_Network) {
  let Action = /*#__PURE__*/function (Action) {
    Action["Request"] = "request";
    Action["Response"] = "response";
    return Action;
  }({});
  _Network.Action = Action;
  class Data {
    constructor(action, url, data) {
      this.action = action;
      this.url = url;
      this.data = data;
    }
  }
  _Network.Data = Data;
  class Signal extends RawSignal {
    constructor(action, url, data) {
      super();
      const json = data === null || data === undefined ? undefined : JSON.parse(data);
      this.data = new Data(action, url, json);
    }
    anonymousId = _signals.Signals.anonymousId;
    type = SignalType.Network;
    timestamp = new Date().toISOString();
    index = _signals.Signals.nextIndex;
  }
  _Network.Signal = Signal;
})(Network || (exports.Network = Network = {}));
let LocalData = exports.LocalData = void 0;
(function (_LocalData) {
  let Action = /*#__PURE__*/function (Action) {
    Action["Loaded"] = "loaded";
    Action["Updated"] = "updated";
    Action["Saved"] = "saved";
    Action["Deleted"] = "deleted";
    Action["Undefined"] = "undefined";
    return Action;
  }({});
  _LocalData.Action = Action;
  class Data {
    constructor(action, identifier, data) {
      this.action = action;
      this.identifier = identifier;
      this.data = data;
    }
  }
  _LocalData.Data = Data;
  class Signal extends RawSignal {
    constructor(action, identifier, data) {
      super();
      const json = data === null || data === undefined ? undefined : JSON.parse(data);
      this.data = new Data(action, identifier, json);
    }
    anonymousId = _signals.Signals.anonymousId;
    type = SignalType.LocalData;
    timestamp = new Date().toISOString();
    index = _signals.Signals.nextIndex;
  }
  _LocalData.Signal = Signal;
})(LocalData || (exports.LocalData = LocalData = {}));
let Instrumentation = exports.Instrumentation = void 0;
(function (_Instrumentation) {
  let EventType = /*#__PURE__*/function (EventType) {
    EventType["Track"] = "track";
    EventType["Screen"] = "screen";
    EventType["Alias"] = "alias";
    EventType["Identify"] = "identify";
    EventType["Group"] = "group";
    EventType["Unknown"] = "unknown";
    return EventType;
  }({});
  _Instrumentation.EventType = EventType;
  class Data {
    constructor(type, rawEvent) {
      this.type = type;
      this.rawEvent = rawEvent;
    }
  }
  _Instrumentation.Data = Data;
  class Signal extends RawSignal {
    constructor(event) {
      super();
      const type = event.type;
      this.data = new Data(type, event);
    }
    anonymousId = _signals.Signals.anonymousId;
    type = SignalType.Instrumentation;
    timestamp = new Date().toISOString();
    index = _signals.Signals.nextIndex;
  }
  _Instrumentation.Signal = Signal;
})(Instrumentation || (exports.Instrumentation = Instrumentation = {}));
//# sourceMappingURL=types.js.map