import { Signals } from './signals';
export class RawSignal {}
export let SignalType = /*#__PURE__*/function (SignalType) {
  SignalType["Interaction"] = "interaction";
  SignalType["Navigation"] = "navigation";
  SignalType["Network"] = "network";
  SignalType["LocalData"] = "localData";
  SignalType["Instrumentation"] = "instrumentation";
  SignalType["UserDefined"] = "userDefined";
  return SignalType;
}({});
export let Navigation;
(function (_Navigation) {
  let Action = /*#__PURE__*/function (Action) {
    Action["Forward"] = "forward";
    Action["Backward"] = "backward";
    Action["Modal"] = "modal";
    Action["Entering"] = "entering";
    Action["Leaving"] = "leaving";
    Action["Page"] = "page";
    Action["Popup"] = "popup";
    return Action;
  }({});
  _Navigation.Action = Action;
  class Data {
    constructor(action, screen) {
      this.action = action;
      this.screen = screen;
    }
  }
  _Navigation.Data = Data;
  class Signal extends RawSignal {
    constructor(action, screen) {
      super();
      this.data = new Data(action, screen);
    }
    anonymousId = Signals.anonymousId;
    type = SignalType.Navigation;
    timestamp = new Date().toISOString();
    index = Signals.nextIndex;
  }
  _Navigation.Signal = Signal;
})(Navigation || (Navigation = {}));
export let Interaction;
(function (_Interaction) {
  class Data {
    constructor(component, title, data) {
      this.component = component;
      this.title = title;
      this.data = data;
    }
  }
  _Interaction.Data = Data;
  class Signal extends RawSignal {
    constructor(control, title, data) {
      super();
      const json = data === null || data === undefined ? undefined : JSON.parse(data);
      this.data = new Data(control, title, json);
    }
    anonymousId = Signals.anonymousId;
    type = SignalType.Interaction;
    timestamp = new Date().toISOString();
    index = Signals.nextIndex;
  }
  _Interaction.Signal = Signal;
})(Interaction || (Interaction = {}));
export let Network;
(function (_Network) {
  let Action = /*#__PURE__*/function (Action) {
    Action["Request"] = "request";
    Action["Response"] = "response";
    return Action;
  }({});
  _Network.Action = Action;
  class Data {
    constructor(action, url, data) {
      this.action = action;
      this.url = url;
      this.data = data;
    }
  }
  _Network.Data = Data;
  class Signal extends RawSignal {
    constructor(action, url, data) {
      super();
      const json = data === null || data === undefined ? undefined : JSON.parse(data);
      this.data = new Data(action, url, json);
    }
    anonymousId = Signals.anonymousId;
    type = SignalType.Network;
    timestamp = new Date().toISOString();
    index = Signals.nextIndex;
  }
  _Network.Signal = Signal;
})(Network || (Network = {}));
export let LocalData;
(function (_LocalData) {
  let Action = /*#__PURE__*/function (Action) {
    Action["Loaded"] = "loaded";
    Action["Updated"] = "updated";
    Action["Saved"] = "saved";
    Action["Deleted"] = "deleted";
    Action["Undefined"] = "undefined";
    return Action;
  }({});
  _LocalData.Action = Action;
  class Data {
    constructor(action, identifier, data) {
      this.action = action;
      this.identifier = identifier;
      this.data = data;
    }
  }
  _LocalData.Data = Data;
  class Signal extends RawSignal {
    constructor(action, identifier, data) {
      super();
      const json = data === null || data === undefined ? undefined : JSON.parse(data);
      this.data = new Data(action, identifier, json);
    }
    anonymousId = Signals.anonymousId;
    type = SignalType.LocalData;
    timestamp = new Date().toISOString();
    index = Signals.nextIndex;
  }
  _LocalData.Signal = Signal;
})(LocalData || (LocalData = {}));
export let Instrumentation;
(function (_Instrumentation) {
  let EventType = /*#__PURE__*/function (EventType) {
    EventType["Track"] = "track";
    EventType["Screen"] = "screen";
    EventType["Alias"] = "alias";
    EventType["Identify"] = "identify";
    EventType["Group"] = "group";
    EventType["Unknown"] = "unknown";
    return EventType;
  }({});
  _Instrumentation.EventType = EventType;
  class Data {
    constructor(type, rawEvent) {
      this.type = type;
      this.rawEvent = rawEvent;
    }
  }
  _Instrumentation.Data = Data;
  class Signal extends RawSignal {
    constructor(event) {
      super();
      const type = event.type;
      this.data = new Data(type, event);
    }
    anonymousId = Signals.anonymousId;
    type = SignalType.Instrumentation;
    timestamp = new Date().toISOString();
    index = Signals.nextIndex;
  }
  _Instrumentation.Signal = Signal;
})(Instrumentation || (Instrumentation = {}));
//# sourceMappingURL=types.js.map