import { SegmentEvent } from '@segment/analytics-react-native';
export type Config = {
    maximumBufferSize: number;
    broadcastInterval: number;
    broadcasters?: SignalBroadcaster[];
};
export declare abstract class RawSignal<T> {
    abstract anonymousId: string;
    abstract type: string;
    abstract timestamp: string;
    abstract index: number;
    abstract data: T;
}
export declare enum SignalType {
    Interaction = "interaction",
    Navigation = "navigation",
    Network = "network",
    LocalData = "localData",
    Instrumentation = "instrumentation",
    UserDefined = "userDefined"
}
export interface SignalBroadcaster {
    added<T>(signal: RawSignal<T>): void;
    relay(): void;
}
export declare namespace Navigation {
    enum Action {
        Forward = "forward",
        Backward = "backward",
        Modal = "modal",
        Entering = "entering",
        Leaving = "leaving",
        Page = "page",
        Popup = "popup"
    }
    class Data {
        readonly action: Action;
        readonly screen: string;
        constructor(action: Action, screen: string);
    }
    class Signal extends RawSignal<Data> {
        constructor(action: Action, screen: string);
        anonymousId: string;
        type: string;
        timestamp: string;
        index: number;
        data: Data;
    }
}
export declare namespace Interaction {
    class Data {
        readonly component: string;
        readonly title?: string;
        readonly data?: object;
        constructor(component: string, title?: string, data?: object);
    }
    class Signal extends RawSignal<Data> {
        constructor(control: string, title?: string, data?: string);
        anonymousId: string;
        type: string;
        timestamp: string;
        index: number;
        data: Data;
    }
}
export declare namespace Network {
    enum Action {
        Request = "request",
        Response = "response"
    }
    class Data {
        readonly action: Action;
        readonly url: string;
        readonly data?: object;
        constructor(action: Action, url: string, data?: object);
    }
    class Signal extends RawSignal<Data> {
        constructor(action: Action, url: string, data?: string);
        anonymousId: string;
        type: string;
        timestamp: string;
        index: number;
        data: Data;
    }
}
export declare namespace LocalData {
    enum Action {
        Loaded = "loaded",
        Updated = "updated",
        Saved = "saved",
        Deleted = "deleted",
        Undefined = "undefined"
    }
    class Data {
        readonly action: Action;
        readonly identifier: string;
        readonly data?: object;
        constructor(action: Action, identifier: string, data?: object);
    }
    class Signal extends RawSignal<Data> {
        constructor(action: Action, identifier: string, data?: string);
        anonymousId: string;
        type: string;
        timestamp: string;
        index: number;
        data: Data;
    }
}
export declare namespace Instrumentation {
    enum EventType {
        Track = "track",
        Screen = "screen",
        Alias = "alias",
        Identify = "identify",
        Group = "group",
        Unknown = "unknown"
    }
    class Data {
        readonly type: EventType;
        readonly rawEvent?: object;
        constructor(type: EventType, rawEvent?: object);
    }
    class Signal extends RawSignal<Data> {
        constructor(event: SegmentEvent);
        anonymousId: string;
        type: string;
        timestamp: string;
        index: number;
        data: Data;
    }
}
//# sourceMappingURL=types.d.ts.map