import { RawSignal, SignalType } from "./types";

export class SignalsJS {

  signalBuffer: RawSignal<any>[] = [];
  signalCounter: number = 0;
  maxBufferSize: number = 1000;

  constructor() {
    this.signalBuffer = []
    this.signalCounter = 0
    this.maxBufferSize = 1000

  }
  
  add(signal: RawSignal<any>) {
      if (this.signalCounter < 0) {
          // we've rolled over?  start back at zero.
          this.signalCounter = 0;
      }
      if (signal.index == -1) {
          signal.index = this.getNextIndex();
      }
      this.signalBuffer.unshift(signal);
      // R-E-S-P-E-C-T that's what this maxBufferSize means to me
      if (this.signalBuffer.length > this.maxBufferSize) {
          this.signalBuffer.pop();
      }
  }

  getNextIndex(): number {
      let index = this.signalCounter;
      this.signalCounter += 1;
      return index;
  }
  
  find(fromSignal: RawSignal<any>, signalType: SignalType, predicate: (s: RawSignal<any>) => boolean): RawSignal<any> | null {
      var fromIndex = 0;
      if (fromSignal != null) {
          this.signalBuffer.find((signal, index) => {
              if (fromSignal === signal) {
                  fromIndex = index;
              }
          });
      }
      
      for (let i = fromIndex; i < this.signalBuffer.length; i++) {
          let s = this.signalBuffer[i];
          if (s && ((s.type === signalType) || (signalType == undefined))) {
              if (predicate != null) {
                  try {
                      if (predicate(s)) {
                          return s;
                      }
                  } catch (e) {  
                  }
              } else {
                  return s;
              }
          }
      }
      
      return null;
  }
  
  findAndApply(fromSignal: RawSignal<any>, signalType: SignalType, searchPredicate: (s: RawSignal<any>) => boolean, applyPredicate: (s: RawSignal<any>) => void): RawSignal<any> | null {
      let result = this.find(fromSignal, signalType, searchPredicate)
      if (result) {
          applyPredicate(result)
      }
      return result
  }
}
