import { PluginType, SegmentClient, UtilityPlugin } from "@segment/analytics-react-native";
import { Signals } from "../signals";
import { Network } from "../types";


const originalFetch = global.fetch;

export class SignalsNetworkTracking extends UtilityPlugin {
  
  type = PluginType.utility;

  configure(analytics: SegmentClient): void {
    super.configure(analytics);

    global.fetch = async (url, data) => {
      const isSegmentEndpoint = this.containsSpecificUrl(url, 'segment.io');
      const urlStr = this.urlToString(url);
      
      // Intercept the request
      if (!isSegmentEndpoint) {
        Signals.emit(new Network.Signal(Network.Action.Request, urlStr, JSON.stringify(data)));
      }
    
      // Perform the original fetch
      const response = await originalFetch(url, data);
    
      // Intercept the response
      const clonedResponse = response.clone();
      const responseBody = await clonedResponse.json();
      if (!isSegmentEndpoint) {
        Signals.emit(new Network.Signal(Network.Action.Response, urlStr, responseBody));
      }
    
      return response;
    };
  }

  containsSpecificUrl(value: URL | RequestInfo, specificUrl: string): boolean {
    if (typeof value === 'string') {
      return value.includes(specificUrl);
    } else if (value instanceof URL) {
      return value.href.includes(specificUrl);
    } else if (value instanceof Request) {
      return value.url.includes(specificUrl);
    }
    
    return false;
  }

  urlToString(value: URL | RequestInfo): string {
    if (typeof value === 'string') {
      return value;
    } else if (value instanceof URL) {
      return value.href;
    } else if (value instanceof Request) {
      return value.url;
    }
    
    return '';
  }
}